#ifndef __G_GLOBALS_INCLUDED__
#define __G_GLOBALS_INCLUDED__

#define COPYRIGHT_STRING "Copyright (c) 2001 Gianni Tedesco" CRLF \
			"Released under the terms of the GNU GPL"
#define VERSION_STRING "Firewall Monitor (fwmon) v1.0.5" CRLF COPYRIGHT_STRING

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>

#include <net/if.h>

#include <arpa/inet.h>

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <stddef.h>

#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include <signal.h>

#define _GNU_SOURCE
#include "g_globals.h"

#define _BSD_SRC
#include <grp.h>

#include "lang.h"

#define BUFFER_SIZE 4096
#define NETLINK_DEV "/dev/netlink"

#define print_err(x, arg...) fprintf(stderr, x, ## arg);

struct icmphdr {
	unsigned char type;
	unsigned char code;
	unsigned short csum;
	unsigned short id;
	unsigned short seq;
}icmphdr;

typedef struct netlink_t {
	unsigned int len;
	unsigned int mark;
	char iface[IFNAMSIZ];
}netlink_t;

typedef struct data_t{
	struct netlink_t netlink;
	unsigned char  data[BUFFER_SIZE];
}data_t;

typedef struct packet_t{
	struct tm *time;
	struct data_t data;
	struct iphdr *ip;
	void *trans;
	void *app;
}packet_t, *p_packet;

#define flag(x) (globals.flags & x)
#define flag_set(x) globals.flags |= x
#define flag_unset(x) globals.flags &= ~x

#define err(x) (globals.error & x)
#define err_set(x) globals.error |= x
#define err_unset(x) globals.error &= ~x

#define FLAG_NONE	0x00000000
#define FLAG_DAEMON	0x00000001	/* Go to daemon mode */
#define FLAG_STDOUT	0x00000002	/* Print to stdout */
#define FLAG_LOGFILE	0x00000004	/* globals.logfile is valid */
#define FLAG_LIBPCAP	0x00000008	/* globals.pcapfile is valid */
#define FLAG_SUID	0x00000010	/* globals.uid is valid */
#define FLAG_SGID	0x00000020	/* globals.gid is valid */
#define FLAG_CHROOT	0x00000040	/* globals.chroot is valid */
#define FLAG_DROPROOT	0x00000080	/* a non root uid/gid is set */
#define FLAG_SHOWDATE	0x00000100	/* show the date on logged lines? */
#define FLAG_NODATA	0x00000200	/* don't dump data */
#define FLAG_NOHEX	0x00000400	/* don't show hex output */
#define FLAG_NOASCII	0x00000800	/* don't show ascii output */


#define ERROR_SUGID	0x00000001	/* uid or gid set, but not both */
#define ERROR_FMATCH	0x00000002	/* pcap and logfile are the same */
#define ERROR_NODATA	0x00000004	/* nohex and noascii */
#define ERROR_ZEROLINE	0x00000008	/* zero line length */

#define LINE_LENGTH 16

typedef struct globals_t {
	unsigned long flags;
	unsigned long error;
	int nosync;
	int uid;
	int gid;
	char *chroot;
	char *logfile;
	char *pcapfile;
	int length;
}globals_t;

extern struct globals_t globals;

#endif //__G_GLOBALS_INCLUDED__
